@@ -49,20 +49,24 @@ class BondMapException : public std::runtime_error
 class BondMap
 {
   public:
-	BondMap(const cif::mm::structure &structure)
-		: BondMap(structure.get_datablock(), structure.get_model_nr())
+	BondMap(const cif::mm::structure &structure, std::optional<std::tuple<cif::point,float>> around = {})
+		: BondMap(structure.get_datablock(), around, structure.get_model_nr())
 	{
-
 	}
 
-	BondMap(const cif::datablock &db, size_t model_nr = 1);
+	BondMap(const cif::datablock &db, std::optional<std::tuple<cif::point,float>> around = {}, size_t model_nr = 1);
 
 	BondMap(const BondMap &) = delete;
 	BondMap &operator=(const BondMap &) = delete;
 
+	BondMap(BondMap &&);
+	BondMap &operator=(BondMap &&);
+
 	bool operator()(const std::string &atom_1, const std::string &atom_2) const
 	{
-		return isBonded(index.at(atom_1), index.at(atom_2));
+		auto aix1 = index.find(atom_1);
+		auto aix2 = index.find(atom_2);
+		return aix1 != index.end() and aix2 != index.end() and isBonded(aix1->second, aix2->second);
 	}
 
 	bool operator()(const cif::mm::atom &atom_1, const cif::mm::atom &atom_2) const
