@@ -29,24 +29,18 @@
    Date: dinsdag 22 mei, 2018
 */
 
-#include <numeric>
-
-#include <cif++.hpp>
-
-#include "pdb-redo/ClipperWrapper.hpp"
 #include "pdb-redo/Minimizer.hpp"
 #include "pdb-redo/Restraints.hpp"
 
+#include <cif++.hpp>
+
+#include <Eigen/Eigenvalues>
+
+#include <numeric>
+
 namespace pdb_redo
 {
 
-using clipper::Coord_frac;
-using clipper::Coord_grid;
-using clipper::Coord_map;
-using clipper::Coord_orth;
-
-// --------------------------------------------------------------------
-
 double BondRestraint::f(const AtomLocationProvider &atoms) const
 {
 	double d = mDist - distance(atoms[mA], atoms[mB]);
@@ -55,7 +49,7 @@ double BondRestraint::f(const AtomLocationProvider &atoms) const
 	if (cif::VERBOSE > 2)
 		std::cerr << "bond::f() = " << atoms.atom(mA) << " <> " << atoms.atom(mB)
 				  << " " << atoms[mA] << " <> " << atoms[mB]
-				  << " => " << result << std::endl;
+				  << " => " << result << '\n';
 
 	return result;
 }
@@ -72,7 +66,7 @@ void BondRestraint::df(const AtomLocationProvider &atoms, DFCollector &df) const
 
 	if (cif::VERBOSE > 2)
 		std::cerr << "bond::df(): " << atoms.atom(mA) << " <> " << atoms.atom(mB) << ' '
-				  << bi << ' ' << mDist << ' ' << mDistESD << std::endl;
+				  << bi << ' ' << mDist << ' ' << mDistESD << '\n';
 
 	df.add(mA, (a1 - a2) * c);
 	df.add(mB, (a2 - a1) * c);
@@ -80,7 +74,7 @@ void BondRestraint::df(const AtomLocationProvider &atoms, DFCollector &df) const
 
 void BondRestraint::print(const AtomLocationProvider &atoms) const
 {
-	std::cout << "bond " << atoms.atom(mA) << " to " << atoms.atom(mB) << " => " << mDist << " / " << mDistESD << std::endl;
+	std::cout << "bond " << atoms.atom(mA) << " to " << atoms.atom(mB) << " => " << mDist << " / " << mDistESD << '\n';
 }
 
 // --------------------------------------------------------------------
@@ -96,7 +90,7 @@ double AngleRestraint::f(const AtomLocationProvider &atoms) const
 	double result = (d * d) / (mESD * mESD);
 
 	if (cif::VERBOSE > 2)
-		std::cerr << "angle::f() " << atoms.atom(mA) << "/" << atoms.atom(mB) << "/" << atoms.atom(mC) << ' ' << " = " << result << std::endl;
+		std::cerr << "angle::f() " << atoms.atom(mA) << "/" << atoms.atom(mB) << "/" << atoms.atom(mC) << ' ' << " = " << result << '\n';
 
 	return result;
 }
@@ -106,7 +100,7 @@ void AngleRestraint::df(const AtomLocationProvider &atoms, DFCollector &df) cons
 	const double kRadToDegree = 180.0 / cif::kPI, kDegreeToRad = 1 / kRadToDegree;
 
 	if (cif::VERBOSE > 2)
-		std::cerr << "angle::df() " << atoms.atom(mA) << "/" << atoms.atom(mB) << "/" << atoms.atom(mC) << ' ' << ": " << std::endl;
+		std::cerr << "angle::df() " << atoms.atom(mA) << "/" << atoms.atom(mB) << "/" << atoms.atom(mC) << ' ' << ": \n";
 
 	DPoint k = atoms[mA], l = atoms[mB], m = atoms[mC];
 
@@ -152,7 +146,7 @@ void AngleRestraint::df(const AtomLocationProvider &atoms, DFCollector &df) cons
 
 void AngleRestraint::print(const AtomLocationProvider &atoms) const
 {
-	std::cout << "angle " << atoms.atom(mA) << " ; " << atoms.atom(mB) << " ; " << atoms.atom(mC) << " => " << mAngle << " / " << mESD << std::endl;
+	std::cout << "angle " << atoms.atom(mA) << " ; " << atoms.atom(mB) << " ; " << atoms.atom(mC) << " => " << mAngle << " / " << mESD << '\n';
 }
 
 // --------------------------------------------------------------------
@@ -243,7 +237,7 @@ double TorsionRestraint::f(const AtomLocationProvider &atoms) const
 					  << " diff: " << diff
 					  << " target: " << mTarget << " sigma: " << mESD
 					  << " atoms: " << atoms.atom(mA) << ", " << atoms.atom(mB) << ", " << atoms.atom(mC) << ", " << atoms.atom(mD)
-					  << std::endl;
+					  << '\n';
 	}
 
 	return result;
@@ -252,7 +246,7 @@ double TorsionRestraint::f(const AtomLocationProvider &atoms) const
 void TorsionRestraint::df(const AtomLocationProvider &atoms, DFCollector &df) const
 {
 	if (cif::VERBOSE > 2)
-		std::cerr << "torsion::df() " << atoms.atom(mA) << "/" << atoms.atom(mB) << "/" << atoms.atom(mC) << "/" << atoms.atom(mD) << ' ' << ": " << std::endl;
+		std::cerr << "torsion::df() " << atoms.atom(mA) << "/" << atoms.atom(mB) << "/" << atoms.atom(mC) << "/" << atoms.atom(mD) << ' ' << ": \n";
 
 	double cos_a1 = cosinus_angle(atoms[mB], atoms[mA], atoms[mC], atoms[mB]);
 	double cos_a2 = cosinus_angle(atoms[mC], atoms[mB], atoms[mD], atoms[mC]);
@@ -287,23 +281,21 @@ void TorsionRestraint::df(const AtomLocationProvider &atoms, DFCollector &df) co
 
 void TorsionRestraint::print(const AtomLocationProvider &atoms) const
 {
-	std::cout << "torsion " << atoms.atom(mA) << " ; " << atoms.atom(mB) << " ; " << atoms.atom(mC) << " ; " << atoms.atom(mD) << " => " << mPeriodicity << " / " << mESD << std::endl;
+	std::cout << "torsion " << atoms.atom(mA) << " ; " << atoms.atom(mB) << " ; " << atoms.atom(mC) << " ; " << atoms.atom(mD) << " => " << mPeriodicity << " / " << mESD << '\n';
 }
 
 // --------------------------------------------------------------------
 
-const double kChiralVolumeESD = 0.2; // according to coot that's a reasonable value...
-
 double ChiralVolumeRestraint::f(const AtomLocationProvider &atoms) const
 {
 	auto chiralVolume = dot_product(atoms[mA1] - atoms[mCentre],
 		cross_product(atoms[mA2] - atoms[mCentre], atoms[mA3] - atoms[mCentre]));
 
 	double d = mVolume - chiralVolume;
-	double result = (d * d) / (kChiralVolumeESD * kChiralVolumeESD);
+	double result = (d * d) / (mESD * mESD);
 
 	if (cif::VERBOSE > 2)
-		std::cerr << "chiral::f() = " << result << std::endl;
+		std::cerr << "chiral::f() = " << result << '\n';
 
 	return result;
 }
@@ -311,7 +303,7 @@ double ChiralVolumeRestraint::f(const AtomLocationProvider &atoms) const
 void ChiralVolumeRestraint::df(const AtomLocationProvider &atoms, DFCollector &df) const
 {
 	if (cif::VERBOSE > 2)
-		std::cerr << "chiral::df(): " << std::endl;
+		std::cerr << "chiral::df(): \n";
 
 	DPoint centre = atoms[mCentre];
 	DPoint a = atoms[mA1] - centre;
@@ -321,7 +313,7 @@ void ChiralVolumeRestraint::df(const AtomLocationProvider &atoms, DFCollector &d
 	auto chiralVolume = dot_product(a, cross_product(b, c));
 
 	auto d = chiralVolume - mVolume;
-	auto s = 2 * d / (kChiralVolumeESD * kChiralVolumeESD);
+	auto s = 2 * d / (mESD * mESD);
 
 	df.add(mCentre, s * DPoint{
 							-(b.m_y * c.m_z - b.m_z * c.m_y) - (a.m_z * c.m_y - a.m_y * c.m_z) - (a.m_y * b.m_z - a.m_z * b.m_y),
@@ -335,7 +327,7 @@ void ChiralVolumeRestraint::df(const AtomLocationProvider &atoms, DFCollector &d
 
 void ChiralVolumeRestraint::print(const AtomLocationProvider &atoms) const
 {
-	std::cout << "chiral volume " << atoms.atom(mA1) << " ; " << atoms.atom(mA2) << " ; " << atoms.atom(mA3) << " => " << mVolume << " / " << kChiralVolumeESD << std::endl;
+	std::cout << "chiral volume " << atoms.atom(mA1) << " ; " << atoms.atom(mA2) << " ; " << atoms.atom(mA3) << " => " << mVolume << " / " << mESD << '\n';
 }
 
 // --------------------------------------------------------------------
@@ -348,26 +340,41 @@ void PlanarityRestraint::calculatePlaneFunction(const AtomLocationProvider &atom
 		center += atoms[a];
 	center /= mAtoms.size();
 
-	clipper::Matrix<double> mat(3, 3);
+	double Cxx = 0, Cyy = 0, Czz = 0, Cxy = 0, Cxz = 0, Cyz = 0;
+
 	for (auto &a : mAtoms)
 	{
-		mat(0, 0) += (atoms[a].m_x - center.m_x) * (atoms[a].m_x - center.m_x);
-		mat(1, 1) += (atoms[a].m_y - center.m_y) * (atoms[a].m_y - center.m_y);
-		mat(2, 2) += (atoms[a].m_z - center.m_z) * (atoms[a].m_z - center.m_z);
-		mat(0, 1) += (atoms[a].m_x - center.m_x) * (atoms[a].m_y - center.m_y);
-		mat(0, 2) += (atoms[a].m_x - center.m_x) * (atoms[a].m_z - center.m_z);
-		mat(1, 2) += (atoms[a].m_y - center.m_y) * (atoms[a].m_z - center.m_z);
+		Cxx += (atoms[a].m_x - center.m_x) * (atoms[a].m_x - center.m_x);
+		Cyy += (atoms[a].m_y - center.m_y) * (atoms[a].m_y - center.m_y);
+		Czz += (atoms[a].m_z - center.m_z) * (atoms[a].m_z - center.m_z);
+		Cxy += (atoms[a].m_x - center.m_x) * (atoms[a].m_y - center.m_y);
+		Cxz += (atoms[a].m_x - center.m_x) * (atoms[a].m_z - center.m_z);
+		Cyz += (atoms[a].m_y - center.m_y) * (atoms[a].m_z - center.m_z);
 	}
+	
+	Eigen::Matrix3d mat;
+	mat << Cxx, Cxy, Cxz,
+		   Cxy, Cyy, Cyz,
+		   Cxz, Cyz, Czz;
 
-	mat(1, 0) = mat(0, 1);
-	mat(2, 0) = mat(0, 2);
-	mat(2, 1) = mat(1, 2);
+	Eigen::EigenSolver<Eigen::Matrix3d> es(mat);
 
-	/*auto eigen = */mat.eigen(true);
+	auto ev = es.eigenvalues();
 
-	abcd[0] = mat(0, 0);
-	abcd[1] = mat(1, 0);
-	abcd[2] = mat(2, 0);
+	float b_ev = std::numeric_limits<float>::max();
+	for (size_t i = 0; i < 3; ++i)
+	{
+		if (ev[i].real() > b_ev)
+			continue;
+
+		b_ev = ev[i].real();
+
+		auto col = es.eigenvectors().col(i);
+
+		abcd[0] = col(0).real();
+		abcd[1] = col(1).real();
+		abcd[2] = col(2).real();
+	}
 
 	double sumSq = 1e-20 + abcd[0] * abcd[0] + abcd[1] * abcd[1] + abcd[2] * abcd[2];
 
@@ -408,7 +415,7 @@ double PlanarityRestraint::f(const AtomLocationProvider &atoms) const
 				return s.str();
 			});
 
-		std::cerr << "plane::f() = " << result << " for " << mAtoms.size() << " atoms " << cif::join(as, ", ") << std::endl;
+		std::cerr << "plane::f() = " << result << " for " << mAtoms.size() << " atoms " << cif::join(as, ", ") << '\n';
 	}
 
 	return result;
@@ -427,7 +434,7 @@ void PlanarityRestraint::df(const AtomLocationProvider &atoms, DFCollector &df)
 				return s.str();
 			});
 
-		std::cerr << "plane::df() for " << mAtoms.size() << " atoms " << cif::join(as, ", ") << std::endl;
+		std::cerr << "plane::df() for " << mAtoms.size() << " atoms " << cif::join(as, ", ") << '\n';
 	}
 
 	double abcd[4];
@@ -450,7 +457,7 @@ void PlanarityRestraint::print(const AtomLocationProvider &atoms) const
 	for (auto &a : mAtoms)
 		std::cout << atoms.atom(a) << ' ';
 
-	std::cout << "=> " << 0 << " / " << mESD << std::endl;
+	std::cout << "=> " << 0 << " / " << mESD << '\n';
 }
 
 // --------------------------------------------------------------------
@@ -464,12 +471,13 @@ double NonBondedContactRestraint::f(const AtomLocationProvider &atoms) const
 	{
 		double d = mMinDist - std::sqrt(distance);
 		result = (d * d) / (mDistESD * mDistESD);
+
+		if (cif::VERBOSE > 2)
+			std::cerr << "non-bonded-contact::f() = " << result << " min-dist is " << mMinDist << " and dist is " << std::sqrt(distance)
+					<< " a1: " << atoms.atom(mA) << " a2: " << atoms.atom(mB) << '\n'
+					<< " a1: " << atoms[mA] << " a2: " << atoms[mB] << '\n';
 	}
 
-	if (cif::VERBOSE > 2)
-		std::cerr << "non-bonded-contact::f() = " << result << " min-dist is " << mMinDist << " and dist is " << std::sqrt(distance)
-				  << " a1: " << atoms.atom(mA) << " a2: " << atoms.atom(mB) << std::endl
-				  << " a1: " << atoms[mA] << " a2: " << atoms[mB] << std::endl;
 
 	return result;
 }
@@ -488,7 +496,7 @@ void NonBondedContactRestraint::df(const AtomLocationProvider &atoms, DFCollecto
 		if (cif::VERBOSE > 2)
 			std::cerr << "non-bonded::df(): " << atoms.atom(mA) << " and " << atoms.atom(mB) << " "
 					  << "distance: " << bi << " "
-					  << "target: " << mMinDist << std::endl;
+					  << "target: " << mMinDist << '\n';
 
 		double c = 2 * (1 - mMinDist / bi) / (mDistESD * mDistESD);
 
@@ -500,7 +508,7 @@ void NonBondedContactRestraint::df(const AtomLocationProvider &atoms, DFCollecto
 void NonBondedContactRestraint::print(const AtomLocationProvider &atoms) const
 {
 	std::cout << "nbc " << atoms.atom(mA) << " " << atoms.atom(mB)
-			  << " => " << distance(atoms[mA], atoms[mB]) << ' ' << mMinDist << " / " << mDistESD << std::endl;
+			  << " => " << distance(atoms[mA], atoms[mB]) << ' ' << mMinDist << " / " << mDistESD << '\n';
 }
 
 // --------------------------------------------------------------------
@@ -519,14 +527,14 @@ double DensityRestraint::f(const AtomLocationProvider &atoms) const
 
 	for (auto &a : mAtoms)
 	{
-		Coord_orth p = toClipper(atoms[a.first]);
-		Coord_frac pf = p.coord_frac(mXMap.cell());
+		clipper::Coord_orth p{atoms[a.first].m_x, atoms[a.first].m_y, atoms[a.first].m_z };
+		clipper::Coord_frac pf = p.coord_frac(mXMap.cell());
 
 		result += a.second * mXMap.interp<clipper::Interp_cubic>(pf);
 	}
 
 	if (cif::VERBOSE > 2)
-		std::cerr << "density::f() = " << -result << std::endl;
+		std::cerr << "density::f() = " << -result << '\n';
 
 	return mMapWeight * -result;
 }
@@ -534,12 +542,12 @@ double DensityRestraint::f(const AtomLocationProvider &atoms) const
 void DensityRestraint::df(const AtomLocationProvider &atoms, DFCollector &df) const
 {
 	if (cif::VERBOSE > 2)
-		std::cerr << "density::df(): " << std::endl;
+		std::cerr << "density::df(): \n";
 
 	for (auto &a : mAtoms)
 	{
-		Coord_orth p = toClipper(atoms[a.first]);
-		Coord_frac pf = p.coord_frac(mXMap.cell());
+		clipper::Coord_orth p{atoms[a.first].m_x, atoms[a.first].m_y, atoms[a.first].m_z };
+		clipper::Coord_frac pf = p.coord_frac(mXMap.cell());
 		auto pm = pf.coord_map(mXMap.grid_sampling());
 
 		clipper::Grad_map<double> grad;
@@ -556,7 +564,7 @@ void DensityRestraint::df(const AtomLocationProvider &atoms, DFCollector &df) co
 
 void DensityRestraint::print(const AtomLocationProvider &atoms) const
 {
-	std::cout << "density " << std::endl;
+	std::cout << "density \n";
 }
 
 } // namespace pdb_redo
\ No newline at end of file
