@@ -32,6 +32,7 @@
 #include "pdb-redo/AtomShape.hpp"
 #include "pdb-redo/BondMap.hpp"
 #include "pdb-redo/ClipperWrapper.hpp"
+#include "pdb-redo/DistanceMap.hpp"
 #include "pdb-redo/Statistics.hpp"
 
 // --------------------------------------------------------------------
@@ -39,10 +40,6 @@
 namespace pdb_redo
 {
 
-using clipper::Coord_grid;
-using clipper::Coord_orth;
-using clipper::Xmap;
-
 using cif::atom_type_traits;
 
 // --------------------------------------------------------------------
@@ -233,13 +230,13 @@ class PointWeightFunction
 
 struct AtomGridData
 {
-	AtomGridData(const Coord_grid &gp, double density)
+	AtomGridData(const clipper::Coord_grid &gp, double density)
 		: p(gp)
 		, density(density)
 	{
 	}
 
-	Coord_grid p;
+	clipper::Coord_grid p;
 	double density;
 };
 
@@ -311,7 +308,7 @@ struct AtomData
 
 // --------------------------------------------------------------------
 
-std::tuple<float, float> CalculateMapStatistics(const Xmap<float> &f)
+std::tuple<float, float> CalculateMapStatistics(const clipper::Xmap<float> &f)
 {
 	double sum = 0, sum2 = 0;
 	int count = 0;
@@ -429,7 +426,7 @@ void StatsCollector::initialize()
 		if (t <= cif::atom_type::He)
 			continue;
 
-		float w = a.get_occupancy() * t;
+		float w = a.get_occupancy() * static_cast<int>(t);
 
 		if (w <= 0)
 			continue;
@@ -462,7 +459,7 @@ void StatsCollector::initialize()
 		if (cif::VERBOSE > 2)
 			std::cerr << (atomData.size() + 1) << '\t'
 					  << atom_type_traits(atom.get_type()).symbol() << '\t'
-					  << radius << std::endl;
+					  << radius << '\n';
 
 		atomData.emplace_back(atom, radius);
 	}
@@ -526,19 +523,19 @@ void StatsCollector::initialize()
 			if (cif::VERBOSE > 1)
 			{
 				swys = dd * (swys - (qa * swy + qb * swxy)) / (ns - 2);
-				std::cerr << std::endl
-						  << "Intercept & gradient before LS: " << qa << " (" << std::sqrt(swys * swxs) << ") " << qb << " (" << std::sqrt(swys * sw) << ')' << std::endl;
+				std::cerr << '\n'
+						  << "Intercept & gradient before LS: " << qa << " (" << std::sqrt(swys * swxs) << ") " << qb << " (" << std::sqrt(swys * sw) << ')' << '\n';
 			}
 
 			qb += 1.0;
 
 			if (cif::VERBOSE > 1)
 			{
-				std::cerr << std::endl
-						  << "Rescale SD(delta-rho) using Q-Q plot for asym " << zsc.first << ':' << std::endl
-						  << std::string(54, '=') << std::endl
-						  << "Input & updated SD(delta-rho): " << mRMSDensityFd << " ; " << qb * mRMSDensityFd << std::endl
-						  << std::endl;
+				std::cerr << '\n'
+						  << "Rescale SD(delta-rho) using Q-Q plot for asym " << zsc.first << ':' << '\n'
+						  << std::string(54, '=') << '\n'
+						  << "Input & updated SD(delta-rho): " << mRMSDensityFd << " ; " << qb * mRMSDensityFd << '\n'
+						  << '\n';
 			}
 		}
 
@@ -548,7 +545,7 @@ void StatsCollector::initialize()
 
 std::vector<ResidueStatistics> StatsCollector::collect() const
 {
-	std::vector<std::tuple<std::string, int, std::string>> residues;
+	residue_list residues;
 	std::vector<cif::mm::atom> atoms;
 
 	for (auto atom : mStructure.atoms())
@@ -569,6 +566,29 @@ std::vector<ResidueStatistics> StatsCollector::collect() const
 	return collect(residues, bbox, true);
 }
 
+std::vector<ResidueStatistics> StatsCollector::collect(const std::string &asymID) const
+{
+	residue_list residues;
+	std::vector<cif::mm::atom> atoms;
+
+	for (auto atom_id : mStructure.get_datablock()["atom_site"].find<std::string>(cif::key("label_asym_id") == asymID, "id"))
+	{
+		auto &atom = atoms.emplace_back(mStructure.get_atom_by_id(atom_id));
+
+		auto k = std::make_tuple(atom.get_label_asym_id(), atom.get_label_seq_id(), atom.get_auth_seq_id());
+
+		if (residues.empty() or residues.back() != k)
+		{
+			residues.emplace_back(move(k));
+			atoms.emplace_back(std::move(atom));
+		}
+	}
+
+	BoundingBox bbox(mStructure, atoms, 5.0f);
+	return collect(residues, bbox, false);
+}
+
+
 std::vector<ResidueStatistics> StatsCollector::collect(const std::string &asymID, int resFirst, int resLast, bool authNameSpace) const
 {
 	residue_list residues;
@@ -660,7 +680,7 @@ std::vector<ResidueStatistics> StatsCollector::collect(const residue_list &resid
 		if (cif::VERBOSE > 2)
 			std::cerr << (atomData.size() + 1) << '\t'
 					  << atom_type_traits(atom.get_type()).symbol() << '\t'
-					  << radius << std::endl;
+					  << radius << '\n';
 
 		atomData.emplace_back(atom, radius);
 	}
@@ -721,8 +741,8 @@ std::vector<ResidueStatistics> StatsCollector::collect(const residue_list &resid
 		}
 		catch (const BondMapException &ex)
 		{
-			std::cerr << "Missing information for compound '" << compID << '\'' << std::endl
-					  << ex.what() << std::endl;
+			std::cerr << "Missing information for compound '" << compID << '\'' << '\n'
+					  << ex.what() << '\n';
 
 			missing.insert(compID);
 		}
@@ -882,7 +902,7 @@ ResidueStatistics StatsCollector::collect(const std::vector<cif::mm::atom> &atom
 		if (cif::VERBOSE > 2)
 			std::cerr << (atomData.size() + 1) << '\t'
 					  << atom_type_traits(atom.get_type()).symbol() << '\t'
-					  << radius << std::endl;
+					  << radius << '\n';
 
 		atomData.emplace_back(atom, radius);
 	}
@@ -951,13 +971,13 @@ void StatsCollector::sumDensity(std::vector<AtomData> &atomData,
 		auto radius = data.radius;
 		double sumDensity = 0;
 
-		iterateGrid(toClipper(atom.get_location()), radius, Fb, [&](Xmap_base::Map_reference_coord &iw)
+		iterateGrid(atom.get_location(), radius, Fb, [&, radius_sq = radius * radius](Xmap_base::Map_reference_coord &iw)
 			{
-			auto p = toPoint(iw.coord_orth());
+			cif::point p = iw.coord_orth();
 			
-			double d = distance(p, atom.get_location());
+			double d = distance_squared(p, atom.get_location());
 
-			if (d <= radius)
+			if (d <= radius_sq)
 			{
 				double density = shape.calculatedDensity(p);
 				
@@ -983,12 +1003,17 @@ void StatsCollector::collectSums(std::vector<AtomData> &atomData, GridPtDataMap
 	const Xmap<float> &Fb = mMapMaker.fb();
 	const Xmap<float> &Fd = mMapMaker.fd();
 
-	cif::Progress progress(atomData.size(), "Stats calculation");
+	cif::progress_bar progress_bar(atomData.size(), "Stats calculation");
 
 	// Iterate over the atom data to collect the sums
 	for (auto &d : atomData)
 	{
-		auto rmsScaledF = mRmsScaled.at(d.asymID);
+		auto rmsi = mRmsScaled.find(d.asymID);
+
+		if (rmsi == mRmsScaled.end())
+			continue;
+
+		auto rmsScaledF = rmsi->second;
 
 		for (auto gp : d.points)
 		{
@@ -1001,7 +1026,7 @@ void StatsCollector::collectSums(std::vector<AtomData> &atomData, GridPtDataMap
 			double e = gp.density / gpd;
 			double t = e * mSZ / rmsScaledF.second;
 
-			Xmap_base::Map_reference_coord ix(Fb, gp.p);
+			clipper::Xmap_base::Map_reference_coord ix(Fb, gp.p);
 
 			double fb = Fb[ix];
 			double fd = Fd[ix];
@@ -1037,7 +1062,7 @@ void StatsCollector::collectSums(std::vector<AtomData> &atomData, GridPtDataMap
 			d.sums.edSums[1] += ed3;
 		}
 
-		progress.consumed(1);
+		progress_bar.consumed(1);
 	}
 }
 
@@ -1052,11 +1077,8 @@ void StatsCollector::calculate(std::vector<AtomData> &atomData) const
 
 // --------------------------------------------------------------------
 
-EDIAStatsCollector::EDIAStatsCollector(MapMaker<float> &mm,
-	cif::mm::structure &structure, bool electronScattering, const BondMap &bondMap)
+EDIAStatsCollector::EDIAStatsCollector(const MapMaker<float> &mm, cif::mm::structure &structure, bool electronScattering)
 	: StatsCollector(mm, structure, electronScattering)
-	, mDistanceMap(structure, mm.spacegroup(), mm.cell(), 3.5f)
-	, mBondMap(bondMap)
 {
 	// create a atom radius map, for EDIA
 
@@ -1089,7 +1111,7 @@ EDIAStatsCollector::EDIAStatsCollector(MapMaker<float> &mm,
 		ediaBFactor = kAverageBFactors[i];
 
 	if (cif::VERBOSE > 1)
-		std::cerr << "Calculating radii with B Factor " << ediaBFactor << std::endl;
+		std::cerr << "Calculating radii with B Factor " << ediaBFactor << '\n';
 
 	for (auto atom : mStructure.atoms())
 	{
@@ -1100,7 +1122,7 @@ EDIAStatsCollector::EDIAStatsCollector(MapMaker<float> &mm,
 		mRadii[atom.get_type()] = shape.radius();
 
 		if (cif::VERBOSE > 1)
-			std::cerr << "Radius for atom with type " << atom_type_traits(atom.get_type()).symbol() << " is " << mRadii[atom.get_type()] << std::endl;
+			std::cerr << "Radius for atom with type " << atom_type_traits(atom.get_type()).symbol() << " is " << mRadii[atom.get_type()] << '\n';
 	}
 }
 
@@ -1108,7 +1130,7 @@ void EDIAStatsCollector::calculate(std::vector<AtomData> &atomData) const
 {
 	StatsCollector::calculate(atomData);
 
-	const Xmap<float> &Fb = mMapMaker.fb();
+	const clipper::Xmap<float> &Fb = mMapMaker.fb();
 	//	Xmap<float>& fd = mMapMaker.fd();
 
 	struct lessAtom
@@ -1120,21 +1142,19 @@ void EDIAStatsCollector::calculate(std::vector<AtomData> &atomData) const
 
 	// Calculate EDIA scores
 
-	cif::Progress progress(atomData.size(), "EDIA calculation");
+	DistanceMap dm(mStructure, 3.5f);
+	BondMap bm = createBondMap(atomData);
+
+	cif::progress_bar progress_bar(atomData.size(), "EDIA calculation");
 
 	for (auto &data : atomData)
 	{
 		auto &atom = data.atom;
 		float radius = mRadii.at(atom.get_type());
 
-		//		if (cif::VERBOSE > 2)
-		//			std::cerr << (atomData.size() + 1) << '\t'
-		//				 << atom_type_traits(atom.get_type()).symbol() << '\t'
-		//				 << radius << std::endl;
-
 		PointWeightFunction w(atom.get_location(), radius);
 
-		std::vector<cif::mm::atom> atomsNearBy = mDistanceMap.near(atom, 3.5f);
+		std::vector<cif::mm::atom> atomsNearBy = dm.near(atom, 3.5f);
 
 		std::vector<PointWeightFunction> wn;
 		for (auto a : atomsNearBy)
@@ -1142,9 +1162,9 @@ void EDIAStatsCollector::calculate(std::vector<AtomData> &atomData) const
 
 		float ediaSum[2] = {};
 
-		iterateGrid(toClipper(atom.get_location()), radius, Fb, [&](auto iw)
+		iterateGrid(atom.get_location(), radius, Fb, [&](auto iw)
 		{
-			cif::point p = toPoint(iw.coord_orth());
+			cif::point p = iw.coord_orth();
 			
 			// EDIA calculations
 			auto fb = Fb[iw];
@@ -1185,7 +1205,7 @@ void EDIAStatsCollector::calculate(std::vector<AtomData> &atomData) const
 				{
 					S.insert(atomsNearBy[i]);
 					
-					if (not mBondMap(atomsNearBy[i], atom))
+					if (not bm(atomsNearBy[i], atom))
 						I.insert(atomsNearBy[i]);
 				}
 			}
@@ -1231,8 +1251,27 @@ void EDIAStatsCollector::calculate(std::vector<AtomData> &atomData) const
 		if (data.edia < 0)
 			data.edia = 0;
 
-		progress.consumed(1);
+		progress_bar.consumed(1);
 	}
 }
 
+BondMap EDIAStatsCollector::createBondMap(std::vector<AtomData> &atomData) const
+{
+	std::vector<cif::point> pts;
+	for (auto a : atomData)
+		pts.emplace_back(a.atom.get_location());
+	
+	cif::point center = cif::centroid(pts);
+	float radius = 0;
+
+	for (auto pt : pts)
+	{
+		auto d = distance(pt, center);
+		if (radius < d)
+			radius = d;
+	}
+
+	return { mStructure.get_datablock(), std::make_tuple(center, radius + 3.5f) };
+}
+
 } // namespace pdb_redo
